@@ -57,7 +57,15 @@ ParamOrGroup = Union[Param, 'ParamGroup']
 
 
 class parameter_action:
+    """
+    Decorator that is used to register :paramref:`Parameter.__init__.action` handler methods to store values that are
+    provided for that type of :class:`Parameter`.  The name of the decorated method is used as the ``action`` name.
+    """
+
     def __init__(self, method: MethodType):
+        """
+        :param method: The method that should be used to handle storing values
+        """
         self.method = method
         update_wrapper(self, method)
 
@@ -90,6 +98,8 @@ class parameter_action:
 
 
 class ParamBase(ABC):
+    """Base class for :class:`Parameter` and :class:`ParamGroup`."""
+
     __name: str = None
     _name: str = None
     group: 'ParamGroup' = None
@@ -99,6 +109,13 @@ class ParamBase(ABC):
     hide: Bool = False
 
     def __init__(self, name: str = None, required: Bool = False, help: str = None, hide: Bool = False):  # noqa
+        """
+        :param name: The name to use for this parameter.  Defaults to the name assigned to this parameter.
+        :param required: Whether this parameter is required or not.  If it is required, then an exception will be
+          raised if the user did not provide a value for this parameter.  Defaults to ``False``.
+        :param help: A brief description of this parameter that will appear in ``--help`` text.
+        :param hide: If ``True``, this parameter will not be included in usage / help messages.  Defaults to ``False``.
+        """
         self.required = required
         self.name = name
         self.help = help
@@ -141,7 +158,11 @@ class ParamBase(ABC):
 
 
 class ParamGroup(ParamBase):
-    """A group of parameters."""
+    """
+    A group of parameters.  Intended to be used as a context manager, where group members are defined inside the
+    ``with`` block.  Allows arbitrary levels of nesting, including mutually dependent groups inside mutually exclusive
+    groups, and vice versa.
+    """
 
     _local = local()
     description: Optional[str]
@@ -159,6 +180,21 @@ class ParamGroup(ParamBase):
         required: Bool = False,
         hide: Bool = False,
     ):
+        """
+        :param name: The name of this group, to appear in help messages.
+        :param description: A brief description for this group, to appear in help messages.
+        :param mutually_exclusive: ``True`` if parameters in this group are mutually exclusive, ``False`` otherwise.
+          I.e., if one parameter in this group is provided, then no other parameter in this group will be allowed.
+          Cannot be specified with ``mutually_dependent``.
+        :param mutually_dependent: ``True`` if parameters in this group are mutually dependent, ``False`` otherwise.
+          I.e., if one parameter in this group is provided, then all other parameters in this group must also be
+          provided.  Cannot be specified with ``mutually_exclusive``.
+        :param required: Whether at least one parameter in this group is required or not.  If it is required, then an
+          exception will be raised if the user did not provide a value for any parameters in this group.  Defaults to
+          ``False``.
+        :param hide: If ``True``, this group of parameters will not be included in usage / help messages.  Defaults to
+          ``False``.
+        """
         super().__init__(name=name, required=required, hide=hide)
         self.description = description
         self.members = []
@@ -341,16 +377,33 @@ class ParamGroup(ParamBase):
 
 
 class Parameter(ParamBase, ABC):
+    """
+    Base class for all other parameters.  It is not meant to be used directly.
+
+    Custom parameter classes should generally extend :class:`BasePositional` or :class:`BaseOption` instead of this,
+    otherwise additional handling may be necessary in the parser.
+    """
+
     _actions: frozenset[str] = frozenset()
     _positional: bool = False
+    _repr_attrs: Optional[Collection[str]] = None
     accepts_none: bool = False
     accepts_values: bool = True
     choices: Optional[Collection[Any]] = None
     metavar: str = None
     nargs: Nargs = Nargs(1)
-    type: Callable = None
+    type: Callable[[str], Any] = None
 
-    def __init_subclass__(cls, accepts_values: bool = None, accepts_none: bool = None):
+    def __init_subclass__(
+        cls, accepts_values: bool = None, accepts_none: bool = None, repr_attrs: Collection[str] = None
+    ):
+        """
+        :param accepts_values: Indicates whether a given subclass of Parameter accepts values, or not.  :class:`Flag`
+          is an example of a class that does not accept values.
+        :param accepts_none: Indicates whether a given subclass of Parameter accepts being specified without a value,
+          like :class:`Flag` and :class:`Counter`.
+        :param repr_attrs: Additional attributes to include in the repr.
+        """
         actions = set(cls._actions)  # Inherit actions from parent
         try:
             actions.update(getattr(cls, f'_{cls.__name__}__actions'))
@@ -363,6 +416,8 @@ class Parameter(ParamBase, ABC):
             cls.accepts_values = accepts_values
         if accepts_none is not None:
             cls.accepts_none = accepts_none
+        if repr_attrs is not None:
+            cls._repr_attrs = repr_attrs
 
     def __init__(
         self,
@@ -375,6 +430,20 @@ class Parameter(ParamBase, ABC):
         help: str = None,  # noqa
         hide: Bool = False,
     ):
+        """
+        :param action: The action to take on individual parsed values.  Actions must be defined as methods in classes
+          that extend Parameter, and must be registered via :class:`parameter_action`.
+        :param name: The name to use for this parameter.  Defaults to the name assigned to this parameter.
+        :param default: The default value for this parameter if it is not specified.  Defaults to ``None`` if this
+          parameter is not required; not used if it is required.
+        :param required: Whether this parameter is required or not.  If it is required, then an exception will be
+          raised if the user did not provide a value for this parameter.  Defaults to ``False``.
+        :param metavar: The name to use as a placeholder for values in usage / help messages.
+        :param choices: A container that holds the specific values that users must pick from.  By default, any value is
+          allowed.
+        :param help: A brief description of this parameter that will appear in ``--help`` text.
+        :param hide: If ``True``, this parameter will not be included in usage / help messages.  Defaults to ``False``.
+        """
         if action not in self._actions:
             raise ParameterDefinitionError(
                 f'Invalid {action=} for {self.__class__.__name__} - valid actions: {sorted(self._actions)}'
@@ -393,6 +462,8 @@ class Parameter(ParamBase, ABC):
 
     def __repr__(self) -> str:
         attrs = ('action', 'const', 'default', 'type', 'choices', 'required', 'hide', 'help')
+        if extra_attrs := self._repr_attrs:
+            attrs = chain(attrs, extra_attrs)
         kwargs = ', '.join(
             f'{a}={v!r}'
             for a in attrs
@@ -541,9 +612,24 @@ class Parameter(ParamBase, ABC):
 
 
 class BasePositional(Parameter, ABC):
+    """
+    Base class for :class:`Positional`, :class:`SubCommand`, :class:`Action`, and any other parameters that are
+    provided positionally, without prefixes.  It is not meant to be used directly.
+
+    All positional parameters are required by default.
+
+    Custom positional parameter classes should extend this class to be treated the same as other positionals by the
+    parser.
+    """
+
     _positional: bool = True
 
     def __init__(self, action: str, **kwargs):
+        """
+        :param action: The action to take on individual parsed values.  Actions must be defined as methods in classes
+          that extend Parameter, and must be registered via :class:`parameter_action`.
+        :param kwargs: Additional keyword arguments to pass to :class:`Parameter`.
+        """
         if not (required := kwargs.setdefault('required', True)):
             cls_name = self.__class__.__name__
             raise ParameterDefinitionError(f'All {cls_name} parameters must be required - invalid {required=}')
@@ -561,14 +647,34 @@ class BasePositional(Parameter, ABC):
 
 
 class Positional(BasePositional):
+    """A parameter that must be provided positionally."""
+
     def __init__(
         self,
         nargs: NargsValue = None,
         action: str = _NotSet,
-        type: Callable = None,  # noqa
+        type: Callable[[str], Any] = None,  # noqa
         default: Any = _NotSet,
         **kwargs,
     ):
+        """
+        :param nargs: The number of values that are expected/required for this parameter.  Defaults to 1.  Use a value
+          that allows 0 values to have the same effect as making this parameter not required (the ``required`` option
+          is not supported for Positional parameters).  Only the last Positional parameter in a given
+          :class:`~.commands.Command` may allow a variable / unbound number of arguments.  See :class:`~.nargs.Nargs`
+          for more info.
+        :param action: The action to take on individual parsed values.  Actions must be defined as methods in classes
+          that extend Parameter, and must be registered via :class:`parameter_action`.  Defaults to ``store`` when
+          ``nargs=1``, and to ``append`` otherwise.  A single value will be stored when ``action='store'``, and a list
+          of values will be stored when ``action='append'``.
+        :param type: A callable (function, class, etc.) that accepts a single string argument, which should be called
+          on every value for this parameter to transform the value.  By default, no transformation is performed, and
+          values will be strings.  If not specified, but a type annotation is detected, then that annotation will be
+          used as if it was provided here.  When both are present, this argument takes precedence.
+        :param default: Only supported when ``action='store'`` and 0 values are allowed by the specified ``nargs``.
+          Defaults to ``None`` under those conditions.
+        :param kwargs: Additional keyword arguments to pass to :class:`BasePositional`.
+        """
         if nargs is not None:
             self.nargs = Nargs(nargs)
             if self.nargs == 0:
@@ -604,6 +710,11 @@ class Positional(BasePositional):
 
 
 class Choice:
+    """
+    Used internally to store a value that can be provided as a choice for a parameter, and the target value / callable
+    associated with that choice.
+    """
+
     __slots__ = ('choice', 'target', 'help')
 
     def __init__(self, choice: Optional[str], target: Any = _NotSet, help: str = None):  # noqa
@@ -622,6 +733,14 @@ class Choice:
 
 
 class ChoiceMap(BasePositional):
+    """
+    Base class for :class:`SubCommand` and :class:`Action`.  It is not meant to be used directly.
+
+    Allows choices to be defined and provided as a string that may contain spaces, without requiring users to escape or
+    quote the string (i.e., as technically separate arguments).  This allows for a more natural way to provide
+    multi-word commands, without needing to jump through hoops to handle them.
+    """
+
     _choice_validation_exc = ParameterDefinitionError
     _default_title: str = 'Choices'
     nargs = Nargs('+')
@@ -630,6 +749,11 @@ class ChoiceMap(BasePositional):
     description: Optional[str]
 
     def __init_subclass__(cls, title: str = None, choice_validation_exc: Type[Exception] = None, **kwargs):
+        """
+        :param title: Default title to use for help text sections containing the choices for this parameter.
+        :param choice_validation_exc: The type of exception to raise when validating defined choices.
+        :param kwargs: Additional keyword arguments to pass to :meth:`Parameter.__init_subclass__`.
+        """
         super().__init_subclass__(**kwargs)
         if title is not None:
             cls._default_title = title
@@ -637,6 +761,17 @@ class ChoiceMap(BasePositional):
             cls._choice_validation_exc = choice_validation_exc
 
     def __init__(self, action: str = 'append', title: str = None, description: str = None, **kwargs):
+        """
+        :param action: The action to take on individual parsed values.  Actions must be defined as methods in classes
+          that extend Parameter, and must be registered via :class:`parameter_action`.  Defaults to (and only supports)
+          ``append``.  The ``nargs`` value is automatically calculated / maintained, based on the number of distinct
+          words in the defined choices.  While most parameters that use ``action='append'`` will return a list, the
+          final value for ChoiceMap parameters will instead be a string of space-separated provided values.
+        :param title: The title to use for help text sections containing the choices for this parameter.  Default value
+          depends on what is provided by subclasses.
+        :param description: The description to be used in help text for this parameter.
+        :param kwargs: Additional keyword arguments to pass to :class:`BasePositional`.
+        """
         if (choices := kwargs.setdefault('choices', None)) is not None:
             raise ParameterDefinitionError(
                 f'Invalid {choices=} - {self.__class__.__name__} choices must be added via register'
@@ -740,7 +875,25 @@ class ChoiceMap(BasePositional):
 
 
 class SubCommand(ChoiceMap, title='Subcommands', choice_validation_exc=CommandDefinitionError):
+    """
+    Used to indicate the position where a choice that results in delegating execution of the program to a sub-command
+    should be provided.
+
+    Sub :class:`~.commands.Command` classes are automatically registered as choices for a SubCommand parameter
+    if they extend the Command that contains a SubCommand parameter instead of extending Command directly.  When
+    automatically registered, the choice will be the lower-case name of the sub command class.  It is possible to
+    :meth:`.register` sub commands explicitly to specify a different choice value.
+    """
+
     def __init__(self, *args, required: bool = True, **kwargs):
+        """
+        :param args: Additional positional arguments to pass to :class:`ChoiceMap`.
+        :param required: Whether this parameter is required or not.  If it is required, then an exception will be
+          raised if the user did not provide a value for this parameter.  Defaults to ``True``.  If not required and
+          not provided, the :meth:`~.commands.Command.main` method for the base :class:`~.commands.Command` that
+          contains this SubCommand will be executed by default.
+        :param kwargs: Additional keyword arguments to pass to :class:`ChoiceMap`.
+        """
         super().__init__(*args, **kwargs)
         self.required = required
         if not required:
@@ -795,6 +948,15 @@ class SubCommand(ChoiceMap, title='Subcommands', choice_validation_exc=CommandDe
 
 
 class Action(ChoiceMap, title='Actions'):
+    """
+    Actions are similar to :class:`SubCommand` parameters, but allow methods in :class:`~.commands.Command` classes to
+    be registered as a callable to be executed based on a user's choice instead of separate sub Commands.
+
+    Actions are better suited for use cases where all of the target functions accept the same arguments.  If target
+    functions require different / additional parameters, then using a :class:`SubCommand` with separate sub
+    :class:`~.commands.Command` classes may make more sense.
+    """
+
     def register_action(
         self, choice: Optional[str], method: MethodType, help: str = None, default: Bool = False  # noqa
     ) -> MethodType:
@@ -864,6 +1026,16 @@ class Action(ChoiceMap, title='Actions'):
 
 
 class BaseOption(Parameter, ABC):
+    """
+    Base class for :class:`Option`, :class:`Flag`, :class:`Counter`, and any other keyword-like parameters that have
+    ``--long`` and ``-short`` prefixes before values.
+
+    Only the handling for processing long/short options and formatting usage of these parameters is provided in this
+    class - it is not meant to be used directly.
+
+    Custom option classes should extend this class to be treated the same as other options by the parser.
+    """
+
     # fmt: off
     _long_opts: set[str]        # --long options
     _short_opts: set[str]       # -short options
@@ -871,6 +1043,13 @@ class BaseOption(Parameter, ABC):
     # fmt: on
 
     def __init__(self, *option_strs: str, action: str, **kwargs):
+        """
+        :param option_strs: The long and/or short option prefixes for this option.  If no long prefixes are specified,
+          then one will automatically be added based on the name assigned to this parameter.
+        :param action: The action to take on individual parsed values.  Actions must be defined as methods in classes
+          that extend Parameter, and must be registered via :class:`parameter_action`.
+        :param kwargs: Additional keyword arguments to pass to :class:`Parameter`.
+        """
         if bad_opts := ', '.join(opt for opt in option_strs if not 0 < opt.count('-', 0, 3) < 3):
             raise ParameterDefinitionError(f"Bad option(s) - must start with '--' or '-': {bad_opts}")
         elif bad_opts := ', '.join(opt for opt in option_strs if opt.endswith('-')):
@@ -921,16 +1100,37 @@ class BaseOption(Parameter, ABC):
 
 
 class Option(BaseOption):
+    """A generic option that can be specified as ``--foo bar`` or by using other similar forms."""
+
     def __init__(
         self,
-        *args,
+        *option_strs: str,
         nargs: NargsValue = None,
         action: str = _NotSet,
         default: Any = _NotSet,
         required: Bool = False,
-        type: Callable = None,  # noqa
+        type: Callable[[str], Any] = None,  # noqa
         **kwargs,
     ):
+        """
+        :param option_strs: The long and/or short option prefixes for this option.  If no long prefixes are specified,
+          then one will automatically be added based on the name assigned to this parameter.
+        :param nargs: The number of values that are expected/required when this parameter is specified.  Defaults to 1.
+          See :class:`~.nargs.Nargs` for more info.
+        :param action: The action to take on individual parsed values.  Actions must be defined as methods in classes
+          that extend Parameter, and must be registered via :class:`parameter_action`.  Defaults to ``store`` when
+          ``nargs=1``, and to ``append`` otherwise.  A single value will be stored when ``action='store'``, and a list
+          of values will be stored when ``action='append'``.
+        :param default: The default value for this parameter if it is not specified.  Defaults to ``None`` if
+          this parameter is not required; not used if it is required.
+        :param required: Whether this parameter is required or not.  If it is required, then an exception will be
+          raised if the user did not provide a value for this parameter.  Defaults to ``False``.
+        :param type: A callable (function, class, etc.) that accepts a single string argument, which should be called
+          on every value for this parameter to transform the value.  By default, no transformation is performed, and
+          values will be strings.  If not specified, but a type annotation is detected, then that annotation will be
+          used as if it was provided here.  When both are present, this argument takes precedence.
+        :param kwargs: Additional keyword arguments to pass to :class:`BaseOption`.
+        """
         if not required and default is _NotSet:
             default = None
         if nargs is not None:
@@ -941,7 +1141,7 @@ class Option(BaseOption):
             action = 'store' if self.nargs == 1 else 'append'
         elif action == 'store' and self.nargs != 1:
             raise ParameterDefinitionError(f'Invalid nargs={self.nargs} for {action=}')
-        super().__init__(*args, action=action, default=default, required=required, **kwargs)
+        super().__init__(*option_strs, action=action, default=default, required=required, **kwargs)
         self.type = type
         if action == 'append':
             self._init_value_factory = list
@@ -956,10 +1156,26 @@ class Option(BaseOption):
 
 
 class Flag(BaseOption, accepts_values=False, accepts_none=True):
+    """A (typically boolean) option that does not accept any values."""
+
     __default_const_map = {True: False, False: True, _NotSet: True}
     nargs = Nargs(0)
 
-    def __init__(self, *args, action: str = 'store_const', default: Any = _NotSet, const: Any = _NotSet, **kwargs):
+    def __init__(
+        self, *option_strs: str, action: str = 'store_const', default: Any = _NotSet, const: Any = _NotSet, **kwargs
+    ):
+        """
+        :param option_strs: The long and/or short option prefixes for this option.  If no long prefixes are specified,
+          then one will automatically be added based on the name assigned to this parameter.
+        :param action: The action to take on individual parsed values.  Actions must be defined as methods in classes
+          that extend Parameter, and must be registered via :class:`parameter_action`.  Defaults to ``store_const``, but
+          accepts ``append_const`` to build a list of the specified constant.
+        :param default: The default value for this parameter if it is not specified.  Defaults to ``False`` when
+          ``const=True`` (the default), and to ``True`` when ``const=False``.  Defaults to ``None`` for any other
+          constant.
+        :param const: The constant value to store/append when this parameter is specified.  Defaults to ``True``.
+        :param kwargs: Additional keyword arguments to pass to :class:`BaseOption`.
+        """
         if const is _NotSet:
             try:
                 const = self.__default_const_map[default]
@@ -968,7 +1184,7 @@ class Flag(BaseOption, accepts_values=False, accepts_none=True):
                 raise ParameterDefinitionError(f"Missing parameter='const' for {cls} with {default=}") from e
         if default is _NotSet:
             default = self.__default_const_map.get(const)  # will be True, False, or None
-        super().__init__(*args, action=action, default=default, **kwargs)
+        super().__init__(*option_strs, action=action, default=default, **kwargs)
         self.const = const
 
     def _init_value_factory(self):
@@ -994,16 +1210,36 @@ class Flag(BaseOption, accepts_values=False, accepts_none=True):
     result = result_value
 
 
-class ActionFlag(Flag):
+class ActionFlag(Flag, repr_attrs=('order', 'before_main')):
+    """A :class:`Flag` that triggers the execution of a function / method / other callable when specified."""
+
     def __init__(
-        self, *args, order: Union[int, float] = 1, func: Callable = None, before_main: Bool = True, **kwargs  # noqa
+        self,
+        *option_strs: str,
+        order: Union[int, float] = 1,
+        func: Callable = None,
+        before_main: Bool = True,  # noqa
+        **kwargs,
     ):
+        """
+        :param option_strs: The long and/or short option prefixes for this option.  If no long prefixes are specified,
+          then one will automatically be added based on the name assigned to this parameter.
+        :param order: The priority / order in which this ActionFlag should be executed, relative to other ActionFlags,
+          if others would also be executed.  Two ActionFlags in a given :class:`~.commands.Command` may not have the
+          same combination of ``before_main`` and ``order`` values.  ActionFlags with lower ``order`` values are
+          executed before those with higher values.  The ``--help`` action is implemented as an ActionFlag with
+          ``order=float('-inf')``.
+        :param func: The function to execute when this parameter is specified.
+        :param before_main: Whether this ActionFlag should be executed before the :meth:`~.commands.Command.main`
+          method or after it.
+        :param kwargs: Additional keyword arguments to pass to :class:`Flag`.
+        """
         # TODO: Test in groups, esp mutually excl/dependent
         expected = {'action': 'store_const', 'default': False, 'const': _NotSet}
         found = {k: kwargs.setdefault(k, v) for k, v in expected.items()}
         if bad := {k: v for k, v in found.items() if expected[k] != v}:
             raise ParameterDefinitionError(f'Unsupported kwargs for {self.__class__.__name__}: {bad}')
-        super().__init__(*args, **kwargs)
+        super().__init__(*option_strs, **kwargs)
         self.func = func
         self.order = order
         self.before_main = before_main
@@ -1035,7 +1271,7 @@ class ActionFlag(Flag):
     def __lt__(self, other: 'ActionFlag') -> bool:
         if not isinstance(other, ActionFlag):
             return NotImplemented
-        return (not self.before_main, self.order, self.name) < (not self.before_main, other.order, other.name)
+        return (not self.before_main, self.order, self.name) < (not other.before_main, other.order, other.name)
 
     def __call__(self, func: Callable):
         if self.func is not None:
@@ -1063,14 +1299,31 @@ after_main = partial(ActionFlag, before_main=False)  #: An ActionFlag that will
 
 
 class Counter(BaseOption, accepts_values=True, accepts_none=True):
+    """
+    A :class:`Flag`-like option that counts the number of times it was specified.  Supports an optional integer value
+    to explicitly increase the stored value by that amount.
+    """
+
     type = int
     nargs = Nargs('?')
 
-    def __init__(self, *args, action: str = 'append', default: int = 0, const: int = 1, **kwargs):
+    def __init__(self, *option_strs: str, action: str = 'append', default: int = 0, const: int = 1, **kwargs):
+        """
+        :param option_strs: The long and/or short option prefixes for this option.  If no long prefixes are specified,
+          then one will automatically be added based on the name assigned to this parameter.
+        :param action: The action to take on individual parsed values.  Defaults to ``append``, and no other actions
+          are supported (unless this class is extended).
+        :param default: The default value for this parameter if it is not specified.  This value is also be used as the
+          initial value that will be incremented when this parameter is specified.  Defaults to ``0``.
+        :param const: The value by which the stored value should increase whenever this parameter is specified.
+          Defaults to ``1``.  If a different ``const`` value is used, and if an explicit value is provided by a user,
+          the user-provided value will be added verbatim - it will NOT be multiplied by ``const``.
+        :param kwargs: Additional keyword arguments to pass to :class:`BaseOption`.
+        """
         vals = {'const': const, 'default': default}
         if bad_types := ', '.join(f'{k}={v!r}' for k, v in vals.items() if not isinstance(v, self.type)):
             raise ParameterDefinitionError(f'Invalid type for parameters (expected int): {bad_types}')
-        super().__init__(*args, action=action, default=default, **kwargs)
+        super().__init__(*option_strs, action=action, default=default, **kwargs)
         self.const = const
 
     def _init_value_factory(self):
@@ -1113,9 +1366,16 @@ class Counter(BaseOption, accepts_values=True, accepts_none=True):
 
 
 class PassThru(Parameter):
+    """Collects all remaining arguments, without processing them.  Must be preceded by ``--`` and a space."""
+
     nargs = Nargs('*')
 
     def __init__(self, action: str = 'store_all', **kwargs):
+        """
+        :param action: The action to take on individual parsed values.  Only ``store_all`` (the default) is supported
+          for this parameter type.
+        :param kwargs: Additional keyword arguments to pass to :class:`Parameter`.
+        """
         super().__init__(action=action, **kwargs)
 
     def take_action(self, values: Collection[str], short_combo: bool = False):
